@@ -27,11 +27,11 @@ r = 0.3
 pi = np.pi
 cos = np.cos
 sin = np.sin
-phi, theta = np.mgrid[0:pi:101j, 0:2*pi:101j]
+phi, theta = np.mgrid[0:pi:101j, 0:2 * pi:101j]
 
-x = r*sin(phi)*cos(theta)
-y = r*sin(phi)*sin(theta)
-z = r*cos(phi)
+x = r * sin(phi) * cos(theta)
+y = r * sin(phi) * sin(theta)
+z = r * cos(phi)
 
 mlab.figure(1, bgcolor=(1, 1, 1), fgcolor=(0, 0, 0), size=(400, 300))
 mlab.clf()
@@ -40,13 +40,13 @@ for n in range(1, 6):
     for m in range(n):
         s = sph_harm(m, n, theta, phi).real
 
-        mlab.mesh(x-m, y-n, z, scalars=s, colormap='jet')
+        mlab.mesh(x - m, y - n, z, scalars=s, colormap='jet')
 
-        s[s<0] *= 0.97
+        s[s < 0] *= 0.97
 
         s /= s.max()
-        mlab.mesh(s*x-m, s*y-n, s*z+1.3, scalars=s, colormap='Spectral')
+        mlab.mesh(s * x - m, s * y - n, s * z + 1.3,
+                  scalars=s, colormap='Spectral')
 
 mlab.view(90, 70, 6.2, (-1.3, -2.9, 0.25))
 mlab.show()
-
