@@ -12,6 +12,7 @@ import numpy
 
 # Enthought library imports.
 from traits.api import TraitError
+from pyface.api import GUI
 
 # Local imports.
 from common import TestCase
@@ -41,6 +42,7 @@ class TestImagePlaneWidget(TestCase):
         c.azimuth(30)
         c.elevation(30)
         s.render()
+        GUI.process_events()
 
     def check(self):
         script = self.script
@@ -119,6 +121,7 @@ class TestImagePlaneWidget(TestCase):
         # Set the scene to a suitable view.
         self.set_view(s)
 
+        GUI.process_events()
         self.check()
 
         ############################################################
@@ -132,6 +135,7 @@ class TestImagePlaneWidget(TestCase):
 
         self.set_view(s)
 
+        GUI.process_events()
         self.check()
 
         # Now deepcopy the source and replace the existing one with
@@ -143,6 +147,7 @@ class TestImagePlaneWidget(TestCase):
 
         self.set_view(s)
 
+        GUI.process_events()
         self.check()
 
         # If we have come this far, we are golden!
