@@ -24,8 +24,8 @@ class CellToPointData(FilterBase):
 
     # The actual TVTK filter that this class manages.
     filter = Instance(tvtk.CellDataToPointData,
-                            args=(), kw={'pass_cell_data':1},
-                            allow_none=False, record=True)
+                      args=(), kw={'pass_cell_data':1},
+                      allow_none=False, record=True)
 
     # Information about what this object can consume/produce.
     input_info = PipelineInfo(datasets=['any'],
@@ -36,27 +36,10 @@ class CellToPointData(FilterBase):
                                attribute_types=['any'],
                                attributes=['any'])
 
-    def update_pipeline(self):
-        # Do nothing if there is no input.
-        inputs = self.inputs
-        if len(inputs) == 0:
-            return
-
-        fil = self.filter
-        input = inputs[0].outputs[0]
-        self.configure_connection(fil, inputs[0])
-        fil.update()
-        # This filter creates different outputs depending on the
-        # input.
-        out_map = {'vtkStructuredGrid': 'structured_grid_output',
-                   'vtkRectilinearGrid': 'rectilinear_grid_output',
-                   'vtkStructuredPoints': 'structured_points_output',
-                   'vtkUnstructuredGrid': 'unstructured_grid_output',
-                   'vtkPolyData': 'poly_data_output',
-                   'vtkImageData': 'image_data_output'}
-        # Find the input data type and pass that to our output..
-        for type in out_map:
-            if input.is_a(type):
-                self._set_outputs([getattr(fil, out_map[type])])
-                break
+    def has_output_port(self):
+        """ The filter has an output port."""
+        return True
 
+    def get_output_object(self):
+        """ Return the output port."""
+        return self.filter.output_port
