@@ -1,8 +1,7 @@
 
 from numpy import c_, zeros, arange
 
-from traits.api import HasStrictTraits, \
-    true, false, Instance
+from traits.api import Bool, HasStrictTraits, Instance
 
 from mayavi.sources.vtk_data_source import VTKDataSource
 from mayavi.sources.array_source import ArraySource
@@ -20,35 +19,35 @@ class DataSourceFactory(HasStrictTraits):
         organisation of the data is given by setting the public traits.
     """
 
-    # Whether the position is implicitely inferred from the array indices
-    position_implicit = false
+    # Whether the position is implicitly inferred from the array indices
+    position_implicit = Bool(False)
 
     # Whether the data is on an orthogonal grid
-    orthogonal_grid = false
+    orthogonal_grid = Bool(False)
 
     # If the data is unstructured
-    unstructured = false
+    unstructured = Bool(False)
 
     # If the factory should attempt to connect the data points
-    connected = true
+    connected = Bool(True)
 
     # The position of the data points
     position_x = ArrayOrNone
     position_y = ArrayOrNone
     position_z = ArrayOrNone
 
-    # Connectivity array. If none, it is implicitely inferred from the array
+    # Connectivity array. If none, it is implicitly inferred from the array
     # indices
     connectivity_triangles = ArrayOrNone
 
     # Whether or not the data points should be connected.
-    lines = false
+    lines = Bool(False)
 
     # The scalar data array
     scalar_data = ArrayOrNone
 
     # Whether there is vector data
-    has_vector_data = false
+    has_vector_data = Bool(False)
 
     # The vector components
     vector_u = ArrayOrNone
@@ -159,7 +158,7 @@ class DataSourceFactory(HasStrictTraits):
         """ Uses all the information given by the user on his data
             structure to figure out the right data structure.
         """
-        self.set(**traits)
+        self.trait_set(**traits)
         if not self.lines:
             if self.position_implicit:
                 self._mk_image_data()
