@@ -26,18 +26,18 @@ TODO:
  * When a node is selected, the actor involved could be highlighted.
 
 """
-# Author: Prabhu Ramachandran <prabhu_r@users.sf.net>
-# Copyright (c) 2005, Enthought, Inc.
+# Author: Prabhu Ramachandran <prabhu@aero.iitb.ac.in>
+# Copyright (c) 2005-2020, Enthought, Inc.
 # License: BSD Style.
 
 # Standard library imports.
 import re
 
 # Enthought library imports.
-from traits.api import HasTraits, Property, Any, Instance, \
-                             Trait, List, Str, Dict, Python
+from traits.api import (HasTraits, Property, Any, Instance, Event,
+                        Trait, List, Str, Dict, Python)
 from traitsui.api import \
-     TreeEditor, TreeNodeObject, ObjectTreeNode, View, Item, Group
+     TreeEditor, TreeNodeObject, ObjectTreeNode, View, Item, Group, VSplit
 from traitsui.menu import Menu, Action
 
 from tvtk.api import tvtk
@@ -130,7 +130,8 @@ class SimpleTreeGenerator(TreeGenerator):
 
         for attribute in ['source', 'get_input', 'input', 'mapper', 'property',
                           'texture', 'text_property', 'volume_property',
-                          'lookup_table', 'producer_port', 'producer']:
+                          'lookup_table', 'producer_port', 'producer',
+                          'get_input_algorithm']:
             if hasattr(obj, attribute):
                 return True
 
@@ -174,20 +175,21 @@ class SimpleTreeGenerator(TreeGenerator):
 
         # Check for sources and inputs.
         if hasattr(obj, 'number_of_sources'):
-            srcs = [obj.get_source(i) \
+            srcs = [obj.get_source(i)
                     for i in range(obj.number_of_sources)]
             _add_kid('source', srcs)
         elif hasattr(obj, 'source'):
             _add_kid('source', obj.source)
 
-        if hasattr(obj, 'get_input'):
+        if hasattr(obj, 'get_input_algorithm'):
             inputs = []
             if hasattr(obj, 'number_of_input_ports'):
-                if obj.number_of_input_ports:
-                    inputs = [obj.get_input(i) \
-                              for i in range(obj.number_of_input_ports)]
-            else:
-                inputs = [obj.get_input(i) \
+                inputs = [obj.get_input_algorithm(i, j)
+                          for i in range(obj.number_of_input_ports)
+                          for j in range(
+                                  obj.get_number_of_input_connections(i))]
+            elif hasattr(obj, 'get_input'):
+                inputs = [obj.get_input(i)
                           for i in range(obj.number_of_inputs)]
             _add_kid('input', inputs)
         elif hasattr(obj, 'input'):
@@ -274,30 +276,23 @@ class FullTreeGenerator(SimpleTreeGenerator):
 
         # Check for sources and inputs.
         if hasattr(obj, 'number_of_sources'):
-            srcs = [obj.get_source(i) \
+            srcs = [obj.get_source(i)
                     for i in range(obj.number_of_sources)]
             _add_kid('source', srcs)
         elif hasattr(obj, 'source'):
             _add_kid('source', obj.source)
 
-        if hasattr(obj, 'get_input'):
+        if hasattr(obj, 'get_input_algorithm'):
             inputs = []
             if hasattr(obj, 'number_of_input_ports'):
-                if obj.number_of_input_ports:
-                    # Sometimes not all the inputs can be retrieved using
-                    # 'get_input', as they may be sources (for instance
-                    # the ProbeFilter).
-                    inputs = list()
-                    for i in range(obj.number_of_input_ports):
-                        try:
-                            inputs.append(obj.get_input(i))
-                        except TypeError:
-                            pass
-                    if not inputs:
-                        inputs = [obj.get_input()]
-            else:
-                inputs = [obj.get_input(i) \
-                          for i in range(obj.number_of_inputs)]
+                inputs = [obj.get_input_algorithm(i, j)
+                          for i in range(obj.number_of_input_ports)
+                          for j in range(
+                                  obj.get_number_of_input_connections(i))]
+            _add_kid('input', inputs)
+        elif hasattr(obj, 'get_input'):
+            inputs = [obj.get_input(i)
+                      for i in range(obj.number_of_inputs)]
             _add_kid('input', inputs)
         elif hasattr(obj, 'input'):
             _add_kid('input', obj.input)
@@ -310,19 +305,12 @@ class FullTreeGenerator(SimpleTreeGenerator):
     def has_children(self, obj):
         """Returns true of the object has children, false if not.  This is
         very specific to tvtk objects."""
-        if isinstance(obj, (tvtk.RenderWindow, tvtk.Renderer,
-                            tvtk.Collection)):
-            return True
-        for attribute in ['source', 'get_input', 'input', 'mapper', 'property',
-                          'texture', 'text_property', 'volume_property',
-                          'lookup_table', 'producer_port', 'producer']:
-            if hasattr(obj, attribute):
-                return True
+        result = super(FullTreeGenerator, self).has_children(obj)
         # FIXME: This is inefficient.  We probably should cache the
         # get_children call.
-        if self.get_children(obj):
+        if not result and self.get_children(obj):
             return True
-        return False
+        return result
 
     ###########################################################################
     # Non-public interface.
@@ -354,7 +342,7 @@ class FullTreeGenerator(SimpleTreeGenerator):
         del methods[0]
 
         # using only the first set of indented values.
-        patn = re.compile("  \S")
+        patn = re.compile(r"  \S")
         for method in methods[:]:
             if patn.match(method):
                 if method.find(":") == -1:
@@ -381,10 +369,10 @@ class FullTreeGenerator(SimpleTreeGenerator):
             methods[i] = strng.split(":")
             method_names.append(methods[i][0])
 
-        if re.match("vtk\w*Renderer", vtk_obj.GetClassName()):
+        if re.match(r"vtk\w*Renderer", vtk_obj.GetClassName()):
             methods.append(["ActiveCamera", ""])
 
-        if re.match("vtk\w*Assembly", vtk_obj.GetClassName()):
+        if re.match(r"vtk\w*Assembly", vtk_obj.GetClassName()):
             methods.append(["Parts", ""])
             methods.append(["Volumes", ""])
             methods.append(["Actors", ""])
@@ -401,7 +389,7 @@ class FullTreeGenerator(SimpleTreeGenerator):
         # the browser.  I check for Source and Input anyway so I dont need
         # them.
         for name in('Output', 'FieldData', 'CellData', 'PointData',
-                    'Source', 'Input', 'ExtentTranslator',
+                    'Source', 'Input', 'ExtentTranslator', 'ProgressText',
                     'Interactor', 'Lights', 'Information', 'Executive'):
             _remove_method(name, methods, method_names)
 
@@ -628,9 +616,17 @@ class PipelineBrowser(HasTraits):
     # object instantiation time.
     root_object = List(TVTKBase)
 
+    selected = Instance(TVTKBase)
+
+    # This is fired when an object has been changed on the UI. Use this when
+    # you do not set the renwins list with the render windows but wish to do
+    # your own thing when the object traits are edited on the UI.
+    object_edited = Event
+
     # Private traits.
     # The root of the tree to display.
     _root = Any
+    _ui = Any
 
     ###########################################################################
     # `object` interface.
@@ -648,12 +644,14 @@ class PipelineBrowser(HasTraits):
 
         """
         super(PipelineBrowser, self).__init__(**traits)
-        self.ui = None
+        self._ui = None
         self.view = None
         if renwin:
             self.renwins.append(renwin)
 
         self._root_object_changed(self.root_object)
+
+    def default_traits_view(self):
         menu = Menu(Action(name='Refresh', action='editor.update_editor'),
                     Action(name='Expand all', action='editor.expand_all'))
         self.menu = menu
@@ -664,17 +662,21 @@ class PipelineBrowser(HasTraits):
                                       editable=False,
                                       orientation='vertical',
                                       hide_root=True,
+                                      on_select=self._on_select,
                                       on_dclick=self._on_dclick)
-        self.view = View(Group(Item(name='_root',
-                                    editor=self.tree_editor,
-                                    resizable=True),
-                               show_labels=False,
-                               show_border=False,
-                               orientation='vertical'),
-                         title='Pipeline browser',
-                         help=False,
-                         resizable=True, undo=False, revert=False,
-                         width=.3, height=.3)
+        view = View(Group(VSplit(Item(name='_root',
+                                      editor=self.tree_editor,
+                                      resizable=True),
+                                 Item(name='selected',
+                                      style='custom',
+                                      resizable=True),
+                                 show_labels=False,
+                                 show_border=False)),
+                    title='Pipeline browser',
+                    help=False,
+                    resizable=True, undo=False, revert=False,
+                    width=.3, height=.3)
+        return view
 
     ###########################################################################
     # `PipelineBrowser` interface.
@@ -684,28 +686,29 @@ class PipelineBrowser(HasTraits):
         `parent` widget is passed, the tree is displayed inside the
         passed parent widget."""
         # If UI already exists, raise it and return.
-        if self.ui and self.ui.control:
+        if self._ui is not None and self._ui.control is not None:
             try:
-                self.ui.control.Raise()
+                self._ui.control.Raise()
             except AttributeError:
                 pass
             else:
                 return
         else:
             # No active ui, create one.
-            if parent:
-                self.ui = self.view.ui(self, parent=parent, kind='subpanel')
+            view = self.default_traits_view()
+            if parent is not None:
+                self._ui = view.ui(self, parent=parent, kind='subpanel')
             else:
-                self.ui = self.view.ui(self, parent=parent)
+                self._ui = view.ui(self, parent=parent)
 
     def update(self):
         """Update the tree view."""
         # This is a hack.
-        if self.ui and self.ui.control:
+        if self._ui is not None and self._ui.control is not None:
             try:
-                ed = self.ui._editors[0]
+                ed = self._ui._editors[0]
                 ed.update_editor()
-                self.ui.control.Refresh()
+                self._ui.control.Refresh()
             except (AttributeError, IndexError):
                 pass
     # Another name for update.
@@ -714,6 +717,7 @@ class PipelineBrowser(HasTraits):
     def render(self):
         """Calls render on all render windows associated with this
         browser."""
+        self.object_edited = True
         for rw in self.renwins:
             rw.render()
 
@@ -771,6 +775,17 @@ class PipelineBrowser(HasTraits):
             object.on_trait_change(self.render)
             ui = object.edit_traits(view=view)
 
+    def _on_select(self, obj):
+        if hasattr(obj, 'object') and hasattr(obj.object, 'edit_traits'):
+            new = obj.object
+            old = self.selected
+            if new != old:
+                self.selected = new
+            if old is not None:
+                old.on_trait_change(self.render, remove=True)
+            if new is not None:
+                new.on_trait_change(self.render)
+
 
 
 ######################################################################
@@ -782,7 +797,7 @@ def main(instantiate_gui=True):
 
     v = ivtk.viewer(browser=False, instantiate_gui=instantiate_gui)
     cs = tvtk.ConeSource()
-    m = tvtk.PolyDataMapper(input=cs.output)
+    m = tvtk.PolyDataMapper(input_connection=cs.output_port)
     a = tvtk.Actor(mapper=m)
     v.scene.add_actor(a)
     v.scene.reset_zoom()
@@ -796,5 +811,5 @@ def main(instantiate_gui=True):
 if __name__ == '__main__':
     from pyface.api import GUI
     gui = GUI()
-    main(instantiate_gui=False)
+    v, b, a = main(instantiate_gui=False)
     gui.start_event_loop()
