@@ -5,7 +5,7 @@ make sure that the generated code works well.
 
 """
 # Author: Prabhu Ramachandran
-# Copyright (c) 2004-2016, Enthought, Inc.
+# Copyright (c) 2004-2020, Enthought, Inc.
 # License: BSD Style.
 
 import unittest
@@ -14,21 +14,18 @@ import weakref
 import sys
 import gc
 import traceback
-import contextlib
 import types
 import inspect
 import re
 import numpy
-import vtk
 
 from tvtk import tvtk_base
-from tvtk.common import get_tvtk_name, configure_input_data, is_version_7
+from tvtk.common import get_tvtk_name, configure_input_data
+from tvtk import vtk_module as vtk
+from numpy.testing import assert_array_equal
 
 from traits.api import TraitError
 
-vtk_major_version = vtk.vtkVersion.GetVTKMajorVersion()
-vtk_minor_version = vtk.vtkVersion.GetVTKMinorVersion()
-
 try:
     from tvtk.api import tvtk
 except ImportError:
@@ -44,10 +41,6 @@ To generate tvtk_classes.zip you must do the following::
 # Only used for testing.
 from tvtk.tvtk_classes import tvtk_helper
 
-PY_VER = sys.version_info[0]
-if PY_VER > 2:
-    long = int
-
 
 def mysum(arr):
     val = arr
@@ -97,17 +90,10 @@ class TestTVTK(unittest.TestCase):
         # Shut off pesky warnings.
         vtk.vtkObject.GlobalWarningDisplayOff()
         o = tvtk.ImageFFT()
-        vtk_ver = vtk.vtkVersion().GetVTKVersion()
-        if vtk_ver[:3] in ['4.2', '4.4']:
-            cached = ['ImageFourierFilter', 'ImageDecomposeFilter',
-                      'ImageIterateFilter', 'ImageToImageFilter',
-                      'ImageSource', 'Source', 'ProcessObject',
-                      'Object', 'ObjectBase']
-        else:
-            cached = ['ImageFourierFilter', 'ImageDecomposeFilter',
-                      'ImageIterateFilter', 'ThreadedImageAlgorithm',
-                      'ImageAlgorithm', 'Algorithm', 'Object',
-                      'ObjectBase']
+        cached = ['ImageFourierFilter', 'ImageDecomposeFilter',
+                    'ImageIterateFilter', 'ThreadedImageAlgorithm',
+                    'ImageAlgorithm', 'Algorithm', 'Object',
+                    'ObjectBase']
 
         for i in cached:
             self.assertIn(i, tvtk_helper._cache)
@@ -134,10 +120,7 @@ class TestTVTK(unittest.TestCase):
         r = tvtk.XMLDataReader()
         self.assertEqual(r.f(), 'f')
         if len(vtk.vtkObjectBase.__bases__) > 0:
-            if is_version_7():
-                expect = ()
-            else:
-                expect = (object,)
+            expect = (object,)
         else:
             expect = tuple()
 
@@ -152,12 +135,8 @@ class TestTVTK(unittest.TestCase):
         # If this works without any problems, we are ok.
         cs = tvtk.ConeSource()
         m = tvtk.PolyDataMapper()
-        if vtk_major_version < 6:
-            m.input = cs.output  # This should work
-            m.input = cs.get_output()  # This should also work.
-        else:
-            m.input_connection = cs.output_port  # This should work.
-            m.set_input_data(cs.get_output())  # This should also work.
+        m.input_connection = cs.output_port  # This should work.
+        m.set_input_data(cs.get_output())  # This should also work.
         a = tvtk.Actor()
         a.mapper = m
         cs.resolution = 36
@@ -179,7 +158,7 @@ class TestTVTK(unittest.TestCase):
             if t in ['representation', 'interpolation']:
                 self.assertEqual(val, getattr(p, t + '_'))
             else:
-                self.assertEqual(val, getattr(p, t))
+                assert_array_equal(val, getattr(p, t))
 
     def test_auto_update(self):
         """Test if traits are updated when the VTK object changes."""
@@ -194,12 +173,6 @@ class TestTVTK(unittest.TestCase):
         obj.SetRepresentationToPoints()
         self.assertEqual(p.representation, 'points')
 
-        # color traits have been made non updateable
-        if vtk_major_version <= 5 and vtk_minor_version < 10:
-            val = (1.0, 1.0, 0.0)
-            obj.SetColor(val)
-            self.assertEqual(p.color, val)
-
         val = (1.0, 0.0, 0.0)
         obj.SetDiffuseColor(val)
         self.assertEqual(p.diffuse_color, val)
@@ -220,7 +193,7 @@ class TestTVTK(unittest.TestCase):
         """Test if the help attribute is correct."""
         n = tvtk.PolyDataNormals()
         t = n.traits()
-        test = t['splitting'].help != t['non_manifold_traversal'].help
+        test = t['splitting'].desc != t['non_manifold_traversal'].desc
         self.assertEqual(test, True)
 
     def test_object_cache(self):
@@ -254,8 +227,7 @@ class TestTVTK(unittest.TestCase):
         # For VTK 5.x this test is inconsistent, hence skipeed for 5.x
         # See http://review.source.kitware.com/#/c/15095/
         ##############################################################
-        if vtk_major_version > 5:
-            self.assertEqual(hash1 != hash(src), True)
+        self.assertEqual(hash1 != hash(src), True)
         self.assertEqual(hash(cs), hash(src))
 
         # Test for a bug with collections and the object cache.
@@ -337,19 +309,14 @@ class TestTVTK(unittest.TestCase):
         self.assertEqual(p.color, (0.5, 0.5, 0.5))
 
     def test_points_lookup(self):
-        """ Test if points can be looked up with both int and long keys.
-            Fixes GH Issue 173.
+        """ Test if points can be looked up.
         """
         points = tvtk.Points()
         points.insert_next_point((0, 1, 2))
         pt = points[0]
         self.assertEqual(pt, (0, 1, 2))
-        ptl = points[long(0)]
-        self.assertEqual(ptl, (0, 1, 2))
         get_pt = points.get_point(0)
         self.assertEqual(get_pt, (0, 1, 2))
-        get_ptl = points.get_point(long(0))
-        self.assertEqual(get_ptl, (0, 1, 2))
 
     def test_cell_array(self):
         """ Test if cell array insertion updates number of cells.
@@ -503,13 +470,7 @@ class TestTVTK(unittest.TestCase):
             self.assertEqual(i, j)
         self.assertEqual(f[-1], 3)
         self.assertEqual(f[0], 0)
-        if sys.version_info[0] > 2:
-            self.assertEqual(repr(f), '[0, 1, 2, 3]')
-        else:
-            if type(f[0]) is long:
-                self.assertEqual(repr(f), '[0L, 1L, 2L, 3L]')
-            else:
-                self.assertEqual(repr(f), '[0, 1, 2, 3]')
+        self.assertEqual(repr(f), '[0, 1, 2, 3]')
         f.append(4)
         f.extend([5, 6])
         self.assertEqual(len(f), 7)
@@ -551,24 +512,15 @@ class TestTVTK(unittest.TestCase):
         z = Junk()
         cs = tvtk.ConeSource()
         m = tvtk.PolyDataMapper()
-        if vtk_major_version < 6:
-            m.on_trait_change(z.f, 'input')
-            m.input = cs.output
-            self.assertEqual(z.data, (m, 'input', None, cs.output))
-            m.input = None
-            self.assertEqual(z.data, (m, 'input', cs.output, None))
-            m.on_trait_change(z.f, 'input', remove=True)
-            m.input = cs.output
-        else:
-            m.on_trait_change(z.f, 'input_connection')
-            m.input_connection = cs.output_port
-            self.assertEqual(
-                z.data, (m, 'input_connection', None, cs.output_port))
-            m.input_connection = None
-            self.assertEqual(
-                z.data, (m, 'input_connection', cs.output_port, None))
-            m.on_trait_change(z.f, 'input_connection', remove=True)
-            m.input_connection = cs.output_port
+        m.on_trait_change(z.f, 'input_connection')
+        m.input_connection = cs.output_port
+        self.assertEqual(
+            z.data, (m, 'input_connection', None, cs.output_port))
+        m.input_connection = None
+        self.assertEqual(
+            z.data, (m, 'input_connection', cs.output_port, None))
+        m.on_trait_change(z.f, 'input_connection', remove=True)
+        m.input_connection = cs.output_port
         a = tvtk.Actor()
         a.on_trait_change(z.f, 'mapper')
         a.on_trait_change(z.f, 'property')
@@ -582,24 +534,12 @@ class TestTVTK(unittest.TestCase):
         # Check if property notification occurs on add_input/remove_input
         a = tvtk.AppendPolyData()
         pd = tvtk.PolyData()
-        if vtk_major_version < 6:
-            a.on_trait_change(z.f, 'input')
-            a.add_input(pd)
-            old, new = None, pd
-            self.assertEqual(z.data, (a, 'input', old, new))
-            a.remove_input(pd)
-            old, new = pd, None
-            self.assertEqual(z.data, (a, 'input', old, new))
-            a.remove_all_inputs()
-            old, new = None, None
-            self.assertEqual(z.data, (a, 'input', old, new))
-        else:
-            a.add_input_data(pd)
-            self.assertEqual(a.input, pd)
-            a.remove_input_data(pd)
-            self.assertEqual(a.input, None)
-            a.remove_all_inputs()
-            self.assertEqual(a.input, None)
+        a.add_input_data(pd)
+        self.assertEqual(a.input, pd)
+        a.remove_input_data(pd)
+        self.assertEqual(a.input, None)
+        a.remove_all_inputs()
+        self.assertEqual(a.input, None)
 
     def test_tuple_array_handling(self):
         """Test if methods can take any sequence rather than only tuples."""
@@ -624,31 +564,13 @@ class TestTVTK(unittest.TestCase):
 
     def test_parent_child_input(self):
         """Case where parent has GetInput and child SetInput."""
-        if (vtk_major_version >= 6 and vtk_minor_version > 1) or \
-            vtk_major_version == 7:
-            vm = tvtk.SmartVolumeMapper()
-        else:
-            vm = tvtk.VolumeTextureMapper2D()
+        vm = tvtk.SmartVolumeMapper()
         # In this case if the wrapping is not done right, the input
         # trait is made read-only which is a bug.  We set the input
         # below to test this.
         configure_input_data(vm, tvtk.ImageData())
         spw = tvtk.StructuredPointsWriter()
-        if vtk_major_version < 6:
-            spw.input = None
-        else:
-            spw.input_connection = None
-
-    @unittest.skipIf(
-        vtk_major_version >= 6, "Setting scalar type is no longer supported on VTK 6")
-    def test_image_data_scalar_type(self):
-        """Does ImageData support all scalar types?.
-        """
-        img = tvtk.ImageData()
-        # There are 22 scalar types in VTK-5.2.  We should be able to
-        # use them all.
-        for i in range(0, 22):
-            img.scalar_type = i
+        spw.input_connection = None
 
     def test_null_string_wrapper(self):
         "Check if a null string default is wrapped as a String trait."
@@ -673,39 +595,21 @@ class TestTVTK(unittest.TestCase):
         tvtk_filter = tvtk.ImageConvolve()
         expected = numpy.arange(9.)
 
-        if vtk_major_version < 6:
-            # Before VTK 6, the ImageConvolve.GetKernel3x3
-            # requires an array as argument
+        # Since VTK 6.x, ImageConvolve.GetKernel3x3
+        # can take either no argument or an array
+        # With that change in the API, kernel3x3
+        # is a Trait (similarly for kernel3x3x3, ...)
+        tvtk_filter.kernel3x3 = expected
 
-            # Set the kernel
-            tvtk_filter.set_kernel3x3(expected)
+        # Get it back
+        result = numpy.empty(9)
+        tvtk_filter._vtk_obj.GetKernel3x3(result)
 
-            # Get it back
-            result = numpy.empty(9)
-            tvtk_filter.get_kernel3x3(result)
+        self.assertTrue(numpy.allclose(result, expected), True)
 
-            self.assertTrue(numpy.allclose(result, expected), True)
-
-            with self.assertRaises(TypeError):
-                # VTK Set method raises the TypeError
-                tvtk_filter.set_kernel3x3(range(2))
-
-        else:
-            # Since VTK 6.x, ImageConvolve.GetKernel3x3
-            # can take either no argument or an array
-            # With that change in the API, kernel3x3
-            # is a Trait (similarly for kernel3x3x3, ...)
-            tvtk_filter.kernel3x3 = expected
-
-            # Get it back
-            result = numpy.empty(9)
-            tvtk_filter._vtk_obj.GetKernel3x3(result)
-
-            self.assertTrue(numpy.allclose(result, expected), True)
-
-            # The shape is validated by Trait
-            with self.assertRaises(TraitError):
-                tvtk_filter.kernel3x3 = range(2)
+        # The shape is validated by Trait
+        with self.assertRaises(TraitError):
+            tvtk_filter.kernel3x3 = range(2)
 
     @skipUnlessTVTKHasattr('DistanceRepresentation2D')
     def test_distance_representation_2d_point1_world_position(self):
@@ -714,42 +618,24 @@ class TestTVTK(unittest.TestCase):
         tvtk_filter = tvtk.DistanceRepresentation2D()
         tvtk_filter.instantiate_handle_representation()
 
-        if vtk_major_version < 6:
-            # Before VTK 6, the DistanceRepresentation2D.GetPoint1WorldPosition
-            # requires an array as argument
+        # Since VTK 6.x, DistanceRepresentation2D.GetPoint1WorldPosition
+        # can take either no argument or an array
+        # With that change in the API, point1_world_position
+        # is a Trait (similarly for point2_world_position ...)
 
-            # Set the kernel
-            expected = (1, 2, 3)
-            tvtk_filter.set_point1_world_position(expected)
+        # Set the position
+        expected = (1, 2, 3)
+        tvtk_filter.point1_world_position = expected
 
-            # Get it back
-            result = numpy.empty(3)
-            tvtk_filter.get_point1_world_position(result)
+        # Get it back
+        result = numpy.empty(3)
+        tvtk_filter._vtk_obj.GetPoint1WorldPosition(result)
 
-            self.assertTrue(numpy.allclose(result, expected), True)
+        self.assertTrue(numpy.allclose(result, expected), True)
 
-            with self.assertRaises(TypeError):
-                # VTK Set method raises the TypeError
-                tvtk_filter.set_point1_world_position(range(2))
-        else:
-            # Since VTK 6.x, DistanceRepresentation2D.GetPoint1WorldPosition
-            # can take either no argument or an array
-            # With that change in the API, point1_world_position
-            # is a Trait (similarly for point2_world_position ...)
-
-            # Set the position
-            expected = (1, 2, 3)
-            tvtk_filter.point1_world_position = expected
-
-            # Get it back
-            result = numpy.empty(3)
-            tvtk_filter._vtk_obj.GetPoint1WorldPosition(result)
-
-            self.assertTrue(numpy.allclose(result, expected), True)
-
-            # The shape is validated by Trait
-            with self.assertRaises(TraitError):
-                tvtk_filter.point1_world_position = range(2)
+        # The shape is validated by Trait
+        with self.assertRaises(TraitError):
+            tvtk_filter.point1_world_position = range(2)
 
     @skipUnlessTVTKHasattr('XOpenGLRenderWindow')
     def test_xopengl_render_window(self):
@@ -766,9 +652,6 @@ class TestTVTK(unittest.TestCase):
         with self.assertRaises(TraitError):
             window.position = 1
 
-    @unittest.skipIf(vtk_major_version < 6,
-                     ('vtkHardwareSelector.PropColorValue is not available '
-                      'for vtk version < 6'))
     @skipUnlessTVTKHasattr('HardwareSelector')
     def test_hardware_selector_prop_color_value(self):
         """ Test that PropColorValue of HardwareSelector works
@@ -834,18 +717,78 @@ class TestTVTK(unittest.TestCase):
         self.assertTrue(hasattr(klass, 'set_matrix'))
         self.assertTrue(hasattr(klass, 'get_matrix'))
 
+    def test_algorithm_methods_are_wrapped(self):
+        x = tvtk.Algorithm()
+        self.assertTrue(hasattr(x, 'get_input_algorithm'))
+        self.assertTrue(callable(x.get_input_algorithm))
+        # It should also expose input_algorithm as a property trait.
+        self.assertTrue(hasattr(x, 'input_algorithm'))
+
+    def test_vtk_python_algorithm_base_is_wrapped(self):
+        from tvtk import vtk_module
+        name = 'VTKPythonAlgorithmBase'
+        if hasattr(vtk_module, name):
+            self.assertTrue(hasattr(tvtk, name))
+            # Should be able to instantiate this wrapped class.
+            getattr(tvtk, name)()
+
+    def test_to_vtk_returns_vtk_object(self):
+        # Given
+        x = tvtk.ContourFilter()
+        # When
+        v = tvtk.to_vtk(x)
+        # Then
+        self.assertEqual(v.GetClassName(), 'vtkContourFilter')
+        self.assertTrue(v is x._vtk_obj)
+
+    def test_to_tvtk_returns_tvtk_object(self):
+        # Given
+        v = vtk.vtkContourFilter()
+        # When
+        x = tvtk.to_tvtk(v)
+        # Then
+        self.assertEqual(x.class_name, 'vtkContourFilter')
+        self.assertTrue(isinstance(x, tvtk_base.TVTKBase))
+        self.assertTrue(isinstance(x, tvtk.ContourFilter))
+        self.assertTrue(v is x._vtk_obj)
+
+    def test_to_tvtk_wraps_subclass_of_vtk(self):
+        # Given
+        class MyAlgorithm(vtk.vtkPythonAlgorithm):
+            pass
+
+        a = MyAlgorithm()
+        # When
+        ta = tvtk.to_tvtk(a)
+
+        # Then
+        self.assertTrue(isinstance(ta, tvtk.PythonAlgorithm))
+
+        # Given
+        class B(MyAlgorithm):
+            pass
+
+        b = B()
+        # When
+        tb = tvtk.to_tvtk(b)
+
+        # Then
+        self.assertTrue(isinstance(tb, tvtk.PythonAlgorithm))
+
 
 # This separates out any tests for the entire module that would affect
 # the functioning of the other tests.
 class TestTVTKModule(unittest.TestCase):
 
-    def tearDown(self):
+    @classmethod
+    def tearDownClass(cls):
         tvtk_helper._cache.clear()
         vtk.vtkObject.GlobalWarningDisplayOn()
 
-    def setUp(self):
+    @classmethod
+    def setUpClass(cls):
         vtk.vtkObject.GlobalWarningDisplayOff()
-        self.names = []
+        cls.names = []
         # Filter the ones that are abstract or not implemented
         for name in dir(vtk):
             if (not name.startswith('vtk') or name.startswith('vtkQt') or
@@ -860,7 +803,7 @@ class TestTVTKModule(unittest.TestCase):
                 except (TypeError, NotImplementedError):
                     continue
                 else:
-                    self.names.append(name)
+                    cls.names.append(name)
 
     def test_all_instantiable(self):
         """Test if all the TVTK classes can be instantiated"""
@@ -870,7 +813,7 @@ class TestTVTKModule(unittest.TestCase):
             tvtk_klass = getattr(tvtk, tvtk_name, None)
             try:
                 tvtk_klass()
-            except TraitError:
+            except (TraitError, KeyError):
                 errors.append(traceback.format_exc())
         if len(errors) > 0:
             message = "Not all classes could be instantiated:\n{0}\n"
@@ -912,6 +855,8 @@ class TestTVTKModule(unittest.TestCase):
                 pass
 
             for trait_name in obj.editable_traits():
+                if trait_name in ['_in_set', '_vtk_obj']:
+                    continue
                 vtk_attr_name = to_camel_case(trait_name)
                 min_value, max_value = get_min_max_value(vtk_klass,
                                                          vtk_attr_name)
@@ -995,12 +940,6 @@ class TestTVTKModule(unittest.TestCase):
         self.assertFalse('QtCore' in output)
         self.assertFalse('wx' in output)
 
-    @unittest.skipIf(PY_VER > 2, "Irrelevant for python 3")
-    def test_unicode_traits(self):
-        reader = tvtk.DelimitedTextReader()
-        self.assertIsInstance(reader.unicode_record_delimiters, unicode)
-        self.assertIsInstance(reader.unicode_string_delimiters, unicode)
-        self.assertIsInstance(reader.unicode_field_delimiters, unicode)
 
 if __name__ == "__main__":
     unittest.main()
