@@ -4,15 +4,14 @@ test_tvtk.py.
 
 """
 # Author: Prabhu Ramachandran
-# Copyright (c) 2004-2015, Enthought, Inc.
+# Copyright (c) 2004-2020, Enthought, Inc.
 # License: BSD Style.
-
+from importlib import reload
 import unittest
 import pickle
 import weakref
 import vtk
 import gc
-import imp
 
 from traits import api as traits
 from tvtk import tvtk_base
@@ -23,33 +22,42 @@ from tvtk.common import get_tvtk_name, camel2enthought
 # testing.
 class Prop(tvtk_base.TVTKBase):
     def __init__(self, obj=None, update=1, **traits):
-        tvtk_base.TVTKBase.__init__(self, vtk.vtkProperty, obj, update, **traits)
+        tvtk_base.TVTKBase.__init__(
+            self, vtk.vtkProperty, obj, update, **traits
+        )
 
     edge_visibility = tvtk_base.false_bool_trait
+
     def _edge_visibility_changed(self, old_val, new_val):
         self._do_change(self._vtk_obj.SetEdgeVisibility, self.edge_visibility_)
 
-    representation = traits.Trait('surface',
-                            tvtk_base.TraitRevPrefixMap({'points': 0, 'wireframe': 1, 'surface': 2}))
+    representation = tvtk_base.RevPrefixMap(
+        {'points': 0, 'wireframe': 1, 'surface': 2},
+        4, 5, default_value='surface')
+
     def _representation_changed(self, old_val, new_val):
         self._do_change(self._vtk_obj.SetRepresentation, self.representation_)
 
     opacity = traits.Trait(1.0, traits.Range(0.0, 1.0))
+
     def _opacity_changed(self, old_val, new_val):
         self._do_change(self._vtk_obj.SetOpacity,
                         self.opacity)
 
     specular_color = tvtk_base.vtk_color_trait((1.0, 1.0, 1.0))
+
     def _specular_color_changed(self, old_val, new_val):
         self._do_change(self._vtk_obj.SetSpecularColor,
                         self.specular_color, 1)
 
     diffuse_color = tvtk_base.vtk_color_trait((1.0, 1.0, 1.0))
+
     def _diffuse_color_changed(self, old_val, new_val):
         self._do_change(self._vtk_obj.SetDiffuseColor,
                         self.diffuse_color, 1)
 
     color = tvtk_base.vtk_color_trait((1.0, 1.0, 1.0))
+
     def _color_changed(self, old_val, new_val):
         self._do_change(self._vtk_obj.SetColor,
                         self.color)
@@ -62,7 +70,6 @@ class Prop(tvtk_base.TVTKBase):
                            ('representation', 'GetRepresentation'))
 
 
-
 class TestTVTKBase(unittest.TestCase):
     def test_tvtk_name(self):
         """Test VTK to TVTK class name conversion."""
@@ -77,9 +84,9 @@ class TestTVTKBase(unittest.TestCase):
         num = ['Zero', 'One', 'Two', 'Three', 'Four', 'Five',
                'Six', 'Seven', 'Eight', 'Nine']
         for i in range(10):
-            vn = 'vtk%dA'%i
+            vn = 'vtk%dA' % i
             tn = get_tvtk_name(vn)
-            self.assertEqual(tn, '%sA'%num[i])
+            self.assertEqual(tn, '%sA' % num[i])
 
     def test_camel2enthought(self):
         """Test CamelCase to Enthought style name conversion."""
@@ -103,9 +110,9 @@ class TestTVTKBase(unittest.TestCase):
         p.edge_visibility = not p.edge_visibility
         p.representation = 'p'
         p.opacity = 0.5
-        p.color = (0,1,0)
-        p.diffuse_color = (1,1,1)
-        p.specular_color = (1,1,0)
+        p.color = (0, 1, 0)
+        p.diffuse_color = (1, 1, 1)
+        p.specular_color = (1, 1, 0)
         for t, g in p._updateable_traits_:
             val = getattr(p._vtk_obj, g)()
             if t == 'representation':
@@ -113,6 +120,21 @@ class TestTVTKBase(unittest.TestCase):
             else:
                 self.assertEqual(val, getattr(p, t))
 
+    def test_wrap_call_is_graceful_on_failure(self):
+        # Given
+        p = Prop()
+
+        # When
+        try:
+            # Make a mistake
+            p._wrap_call(p._vtk_obj.SetLineWidth, 'a')
+        except TypeError:
+            pass
+
+        # Then
+        # The _in_set should be reset to zero.
+        self.assertEqual(p._in_set, 0)
+
     def test_auto_update(self):
         """Test trait updation when the VTK object changes."""
         p = Prop()
@@ -176,9 +198,9 @@ class TestTVTKBase(unittest.TestCase):
         p.edge_visibility = 1
         p.representation = 'p'
         p.opacity = 0.5
-        p.color = (0,1,0)
-        p.diffuse_color = (0,1,1)
-        p.specular_color = (1,1,0)
+        p.color = (0, 1, 0)
+        p.diffuse_color = (0, 1, 1)
+        p.specular_color = (1, 1, 0)
 
         s = pickle.dumps(p)
         del p
@@ -213,15 +235,50 @@ class TestTVTKBase(unittest.TestCase):
     def test_rev_prefix_map(self):
         """Test the reverse prefix map trait we use."""
         p = Prop()
+        self.assertEqual(p.representation, 'surface')
+        self.assertEqual(p.representation_, 2)
+
         p.representation = 'p'
+        self.assertEqual(p.representation, 'points')
+        self.assertEqual(p.representation_, 0)
+
         p.representation = 'wire'
+        self.assertEqual(p.representation, 'wireframe')
+        self.assertEqual(p.representation_, 1)
+
         p.representation = 'points'
+        self.assertEqual(p.representation, 'points')
+        self.assertEqual(p.representation_, 0)
+
         p.representation = 2
         self.assertEqual(p.representation, 'surface')
-        self.assertRaises(traits.TraitError, setattr , p,
+        self.assertEqual(p.representation_, 2)
+
+        self.assertRaises(traits.TraitError, setattr, p,
                           'representation', 'points1')
-        self.assertRaises(traits.TraitError, setattr , p,
+        self.assertRaises(traits.TraitError, setattr, p,
                           'representation', 'POINTS')
+        self.assertRaises(traits.TraitError, setattr, p,
+                          'representation', 3)
+
+        with self.assertRaises(traits.TraitError) as exception_context:
+            p.representation = "q"
+        expected_info_text = (
+            "'points' or 'surface' or 'wireframe' (or any unique prefix) "
+            "or 0 or 1 or 2"
+        )
+        self.assertIn(
+            expected_info_text,
+            str(exception_context.exception),
+        )
+
+        # test_zz_object_cache will fail unless we delete our extra reference
+        del exception_context
+
+        # Test extra values
+        p.representation = 4
+        self.assertEqual(p.representation, "surface")
+        self.assertEqual(p.representation_, 2)
 
     def test_deref_vtk(self):
         """Test the `deref_vtk` function."""
@@ -242,6 +299,25 @@ class TestTVTKBase(unittest.TestCase):
         del p
         self.assertEqual(ref(), None)
 
+    def test_global_disable_update(self):
+        # Given
+        p = Prop()
+        vp = tvtk_base.deref_vtk(p)
+
+        # When
+        with tvtk_base.global_disable_update():
+            vp.SetOpacity(0.5)
+            vp.Modified()
+
+        # Then
+        self.assertEqual(p.opacity, 1.0)
+
+        # When
+        vp.SetOpacity(0.4)
+
+        # Then
+        self.assertEqual(p.opacity, 0.4)
+
     def test_strict_traits(self):
         """Test if TVTK objects use strict traits."""
         p = Prop()
@@ -251,7 +327,7 @@ class TestTVTKBase(unittest.TestCase):
 
     def test_init_traits(self):
         """Test if the objects traits can be set in __init__."""
-        p = Prop(opacity=0.1, color=(1,0,0), representation='p')
+        p = Prop(opacity=0.1, color=(1, 0, 0), representation='p')
         self.assertEqual(p.opacity, 0.1)
         self.assertEqual(p.color, (1.0, 0.0, 0.0))
         self.assertEqual(p.representation, 'points')
@@ -272,14 +348,14 @@ class TestTVTKBase(unittest.TestCase):
         self.assertEqual(p, tvtk_base._object_cache.get(addr))
 
         del p
-        gc.collect() # Force collection.
+        gc.collect()  # Force collection.
         self.assertEqual(l1, len(tvtk_base._object_cache))
         self.assertEqual(addr in tvtk_base._object_cache, False)
 
         # Check reload-safety.
         p = Prop()
         l1 = len(tvtk_base._object_cache)
-        imp.reload(tvtk_base)
+        reload(tvtk_base)
         self.assertEqual(l1, len(tvtk_base._object_cache))
 
     # Reloading causes havoc with nosetests based tests so we skip in
